/*
 * Copyright (C) 2019 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 */
package org.securedroneid.android.app;

import androidx.lifecycle.ViewModelProvider;

import android.content.res.Resources;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import org.securedroneid.android.Constants;
import org.securedroneid.android.R;
import org.securedroneid.android.data.Identification;
import org.securedroneid.android.data.MessageData;

import android.graphics.Color;

import java.util.Locale;

public class DeviceDetailFragment extends DialogFragment {
    private static final String TAG = "DeviceDetailFragment";
    private TextView msgVersion;
    private TextView receiveTime;
    private TextView conMac;
    private TextView conRssi;
    private TextView conStarted;
    private TextView conLastUpdate;
    private TextView conMsgDelta;
    private TextView distance;

    private TextView infoLastUpdate1;
    private TextView infoType1;
    private TextView infoIdType1;
    private TextView infoUasId1;
    private ImageView infoVerifiedId1;

    private TextView infoLastUpdate2;
    private TextView infoType2;
    private TextView infoIdType2;
    private TextView infoUasId2;
    private ImageView infoVerifiedId2;

    private TextView posLastUpdate;
    private TextView status;
    private TextView heightType;
    private TextView direction;
    private TextView horiSpeed;
    private TextView vertSpeed;
    private TextView lat;
    private TextView lon;
    private TextView altitudePressure;
    private TextView altitudeGeodetic;
    private TextView height;
    private TextView horizontalAccuracy;
    private TextView verticalAccuracy;
    private TextView baroAccuracy;
    private TextView speedAccuracy;
    private TextView timestamp;
    private TextView timeAccuracy;
    private ImageView locationVerified;

    private TextView dnsOwner;
    private TextView dnsModel;

    private TextView manifestMessageCount;
    private TextView manifestMessageTypes;

    private TextView wrapperMessageCount;
    private TextView wrapperMessageTypes;

    private TextView linkPublicKey;

    private TextView authLastUpdate;
    private TextView authType;
    private TextView authTimestamp;
    private TextView authLength;
    private TextView authData;

    private TextView selfIdLastUpdate;
    private TextView selfIdType;
    private TextView selfIdDescription;
    private ImageView selfIDVerified;

    private TextView systemLastUpdate;
    private TextView operatorLocationType;
    private TextView classificationType;
    private TextView systemLatitude;
    private TextView systemLongitude;
    private TextView systemAreaCount;
    private TextView systemAreaRadius;
    private TextView systemAreaCeiling;
    private TextView systemAreaFloor;
    private TextView category;
    private TextView classValue;
    private TextView systemAltitudeGeo;
    private TextView systemTimestamp;
    private ImageView systemVerified;

    private TextView operatorIdLastUpdate;
    private TextView operatorIdType;
    private TextView operatorId;
    private ImageView operatorVerified;

    static DeviceDetailFragment newInstance() {
        return new DeviceDetailFragment();
    }

    private void setUasIdText(Identification identification, TextView infoUasId) {
        if (identification.getUasIdAsString().length() > Constants.MAX_ID_BYTE_SIZE)
            infoUasId.setTextSize(10);
        else
            infoUasId.setTextSize(14);
        infoUasId.setText(identification.getUasIdAsString());
    }

    private void setVerified(MessageData message, ImageView image) {
        if(message.isVerified()) {
            image.setVisibility(View.VISIBLE);
        }
        else image.setVisibility(View.INVISIBLE);
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        if (getActivity() == null)
            return;

        super.onActivityCreated(savedInstanceState);
        DetailViewModel model = new ViewModelProvider(getActivity()).get(DetailViewModel.class);

        model.connection.observe(getViewLifecycleOwner(), connection -> {
            if (connection == null) return;
            String combo = connection.rssi + " dBm, " + connection.transportType;
            conRssi.setText(combo);
            conMac.setText(connection.macAddress);
            msgVersion.setText(connection.getMsgVersionAsString());
            if (connection.msgVersionUnsupported())
                msgVersion.setTextColor(Color.RED);
            else
                msgVersion.setTextColor(Color.GRAY);
            receiveTime.setText(connection.getTimestampAsString());
            conStarted.setText(String.format(Locale.US,"%s ago", DeviceList.elapsed(connection.firstSeen)));
            conLastUpdate.setText(String.format(Locale.US,"%s ago", DeviceList.elapsed(connection.lastSeen)));
            conMsgDelta.setText(connection.getMsgDeltaAsString());
        });

        model.identification1.observe(getViewLifecycleOwner(), identification -> {
            if (identification == null) return;

            receiveTime.setText(identification.getTimestampAsString());
            infoLastUpdate1.setText(identification.getMsgCounterAsString());
            infoType1.setText(identification.getUaType().toString());
            infoIdType1.setText(identification.getIdType().toString());
            setUasIdText(identification, infoUasId1);
            Log.i(TAG,"identification" + infoUasId1);
            Log.i(TAG,"infouas1" + identification.getIdType().toString());

            setVerified(identification, infoVerifiedId1);
        });

        model.identification2.observe(getViewLifecycleOwner(), identification -> {
            if (identification == null) return;

            receiveTime.setText(identification.getTimestampAsString());
            infoLastUpdate2.setText(identification.getMsgCounterAsString());
            infoType2.setText(identification.getUaType().toString());
            infoIdType2.setText(identification.getIdType().toString());
            setUasIdText(identification, infoUasId2);
            setVerified(identification, infoVerifiedId2);
        });

        model.location.observe(getViewLifecycleOwner(), locationData -> {
            if (locationData == null) return;
            Resources res = getResources();

            receiveTime.setText(locationData.getTimestampAsString());
            posLastUpdate.setText(locationData.getMsgCounterAsString());
            status.setText(locationData.getStatus().toString());
            direction.setText(locationData.getDirectionAsString(res));
            horiSpeed.setText(locationData.getSpeedHorizontalAsString(res));
            vertSpeed.setText(locationData.getSpeedVerticalAsString(res));
            lat.setText(locationData.getLatitudeAsString(res));
            lon.setText(locationData.getLongitudeAsString(res));
            altitudePressure.setText(locationData.getAltitudePressureAsString(res));
            altitudeGeodetic.setText(locationData.getAltitudeGeodeticAsString(res));
            heightType.setText(locationData.getHeightType().toString());
            height.setText(locationData.getHeightAsString(res));
            horizontalAccuracy.setText(locationData.getHorizontalAccuracyAsString(res));
            verticalAccuracy.setText(locationData.getVerticalAccuracyAsString(locationData.getVerticalAccuracy(), res));
            baroAccuracy.setText(locationData.getVerticalAccuracyAsString(locationData.getBaroAccuracy(), res));
            speedAccuracy.setText(locationData.getSpeedAccuracyAsString(res));
            timestamp.setText(locationData.getLocationTimestampAsString());
            timeAccuracy.setText(locationData.getTimeAccuracyAsString(res));
            distance.setText(locationData.getDistanceAsString());
            setVerified(locationData, locationVerified);
        });

        model.dns.observe(getViewLifecycleOwner(), dns -> {
            if (dns == null) return;

            dnsModel.setText(dns.getAbbr());
        });

        model.manifest.observe(getViewLifecycleOwner(), manifest -> {
            if(manifest == null) return;

            receiveTime.setText(manifest.getTimestampAsString());
            manifestMessageTypes.setText(manifest.getMessageTypesAsString());
            manifestMessageCount.setText(String.valueOf(manifest.getMessageCount()));
        });

        model.wrapper.observe(getViewLifecycleOwner(), wrapper -> {
            if(wrapper == null) return;

            receiveTime.setText(wrapper.getTimestampAsString());
            wrapperMessageTypes.setText(wrapper.getMessageTypesAsString());
            wrapperMessageCount.setText(String.valueOf(wrapper.getMessageCount()));
        });

        model.link.observe(getViewLifecycleOwner(), link -> {
            if (link == null) return;

            receiveTime.setText(link.getTimestampAsString());
            linkPublicKey.setText(link.getPublicKeyAsString());
        });

        model.authentication.observe(getViewLifecycleOwner(), authenticationData -> {
            if (authenticationData == null) return;

            Resources res = getResources();
            receiveTime.setText(authenticationData.getTimestampAsString());
            authLastUpdate.setText(authenticationData.getMsgCounterAsString());
            authType.setText(authenticationData.getAuthType().toString());
            authLength.setText(authenticationData.getAuthLengthAsString());
            authTimestamp.setText(authenticationData.getAuthTimestampAsString(res));
            authData.setText(authenticationData.getAuthenticationDataAsString());
        });

        model.selfid.observe(getViewLifecycleOwner(), selfIdData -> {
            if (selfIdData == null) return;

            receiveTime.setText(selfIdData.getTimestampAsString());
            selfIdLastUpdate.setText(selfIdData.getMsgCounterAsString());
            selfIdType.setText(String.valueOf(selfIdData.getDescriptionType().toString()));
            selfIdDescription.setText(selfIdData.getOperationDescriptionAsString());
            setVerified(selfIdData, selfIDVerified);
        });

        model.system.observe(getViewLifecycleOwner(), systemData -> {
            if (systemData == null) return;
            Resources res = getResources();

            receiveTime.setText(systemData.getTimestampAsString());
            systemLastUpdate.setText(systemData.getMsgCounterAsString());
            operatorLocationType.setText(systemData.getOperatorLocationType().toString());
            classificationType.setText(systemData.getclassificationType().toString());
            systemLatitude.setText(systemData.getOperatorLatitudeAsString(res));
            systemLongitude.setText(systemData.getOperatorLongitudeAsString(res));
            systemAreaCount.setText(String.valueOf(systemData.getAreaCount()));
            systemAreaRadius.setText(systemData.getAreaRadiusAsString());
            systemAreaCeiling.setText(systemData.getAreaCeilingAsString(res));
            systemAreaFloor.setText(systemData.getAreaFloorAsString(res));
            category.setText(systemData.getCategory().toString());
            classValue.setText(systemData.getClassValue().toString());
            systemAltitudeGeo.setText(systemData.getOperatorAltitudeGeoAsString(res));
            systemTimestamp.setText(systemData.getSystemTimestampAsString(res));
            setVerified(systemData, systemVerified);
        });

        model.operatorid.observe(getViewLifecycleOwner(), operatorIdData -> {
            if (operatorIdData == null) return;

            receiveTime.setText(operatorIdData.getTimestampAsString());
            operatorIdLastUpdate.setText(operatorIdData.getMsgCounterAsString());
            operatorIdType.setText(String.valueOf(operatorIdData.getOperatorIdType()));
            operatorId.setText(operatorIdData.getOperatorIdAsString());
            setVerified(operatorIdData, operatorVerified);
        });
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.aircraft_details, container, false);
        msgVersion = view.findViewById(R.id.msgVersion);
        receiveTime = view.findViewById(R.id.receiveTime);
        conMac = view.findViewById(R.id.conMac);
        conRssi = view.findViewById(R.id.conRssi);
        conStarted = view.findViewById(R.id.conStarted);
        conLastUpdate = view.findViewById(R.id.conLastUpdate);
        conMsgDelta = view.findViewById(R.id.conMsgDelta);
        distance = view.findViewById(R.id.distance);

        infoLastUpdate1 = view.findViewById(R.id.infoLastUpdate1);
        infoType1 = view.findViewById(R.id.infoType1);
        infoIdType1 = view.findViewById(R.id.infoIdType1);
        infoUasId1 = view.findViewById(R.id.infoUasId1);
        infoVerifiedId1 = view.findViewById(R.id.infoVerifiedId1);

        infoLastUpdate2 = view.findViewById(R.id.infoLastUpdate2);
        infoType2 = view.findViewById(R.id.infoType2);
        infoIdType2 = view.findViewById(R.id.infoIdType2);
        infoUasId2 = view.findViewById(R.id.infoUasId2);
        infoVerifiedId2 = view.findViewById(R.id.infoVerifiedId2);

        posLastUpdate = view.findViewById(R.id.posLastUpdate);
        status = view.findViewById(R.id.status);
        direction = view.findViewById(R.id.direction);
        horiSpeed = view.findViewById(R.id.horiSpeed);
        vertSpeed = view.findViewById(R.id.vertSpeed);
        lat = view.findViewById(R.id.lat);
        lon = view.findViewById(R.id.lon);
        altitudePressure = view.findViewById(R.id.altitudePressure);
        altitudeGeodetic = view.findViewById(R.id.altitudeGeodetic);
        heightType = view.findViewById(R.id.heightType);
        height = view.findViewById(R.id.height);
        horizontalAccuracy = view.findViewById(R.id.horizontalAccuracy);
        verticalAccuracy = view.findViewById(R.id.verticalAccuracy);
        baroAccuracy = view.findViewById(R.id.baroAccuracy);
        speedAccuracy = view.findViewById(R.id.speedAccuracy);
        timestamp = view.findViewById(R.id.timestamp);
        timeAccuracy = view.findViewById(R.id.timeAccuracy);
        locationVerified = view.findViewById(R.id.locationVerified);

        dnsOwner = view.findViewById(R.id.dnsOwner);
        dnsModel = view.findViewById(R.id.dnsModel);

        manifestMessageCount = view.findViewById(R.id.manifestMessageCount);
        manifestMessageTypes = view.findViewById(R.id.manifestMessageTypes);

        wrapperMessageCount = view.findViewById(R.id.wrapperMessageCount);
        wrapperMessageTypes = view.findViewById(R.id.wrapperMessageTypes);

        linkPublicKey = view.findViewById(R.id.publicKey);

        authLastUpdate = view.findViewById(R.id.authLastUpdate);
        authType = view.findViewById(R.id.authType);
        authLength = view.findViewById(R.id.authLength);
        authTimestamp = view.findViewById(R.id.authTimestamp);
        authData = view.findViewById(R.id.authData);

        selfIdLastUpdate = view.findViewById(R.id.selfIdLastUpdate);
        selfIdType = view.findViewById(R.id.selfIdType);
        selfIdDescription = view.findViewById(R.id.selfIdDescription);
        selfIDVerified = view.findViewById(R.id.selfIDVerified);

        systemLastUpdate = view.findViewById(R.id.systemLastUpdate);
        operatorLocationType = view.findViewById(R.id.operatorLocationType);
        classificationType = view.findViewById(R.id.classificationType);
        systemLatitude = view.findViewById(R.id.systemLatitude);
        systemLongitude = view.findViewById(R.id.systemLongitude);
        systemAreaCount = view.findViewById(R.id.systemAreaCount);
        systemAreaRadius = view.findViewById(R.id.systemAreaRadius);
        systemAreaCeiling = view.findViewById(R.id.systemAreaCeiling);
        systemAreaFloor = view.findViewById(R.id.systemAreaFloor);
        category = view.findViewById(R.id.category);
        classValue = view.findViewById(R.id.classValue);
        systemAltitudeGeo = view.findViewById(R.id.systemAltitudeGeo);
        systemTimestamp = view.findViewById(R.id.systemTimestamp);
        systemVerified = view.findViewById(R.id.systemVerified);

        operatorIdLastUpdate = view.findViewById(R.id.operatorIdLastUpdate);
        operatorIdType = view.findViewById(R.id.operatorIdType);
        operatorId = view.findViewById(R.id.operatorId);
        operatorVerified = view.findViewById(R.id.operatorVerified);

        return view;
    }
}
