package org.securedroneid.android.bluetooth;

import static org.securedroneid.android.Constants.SIGNATURE_SIZE;

import android.util.Log;

import androidx.annotation.Nullable;

import org.bouncycastle.math.ec.rfc8032.Ed25519;
import org.securedroneid.android.Constants;

import java.io.StringWriter;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Objects;

public class DRIPLinkObject {
    static String TAG = "DRIPLinkObject";
    Boolean isVerified;
    int VNB;
    int VNA;
    byte[] childDET;
    byte[] parentDET;
    byte[] signature;
    byte[] childHI;
    byte[] parentHI;

    private final byte[] entireMessage;
    private final int messageSize;

    DRIPLinkObject(ByteBuffer linkData){

        this.messageSize = linkData.remaining() - SIGNATURE_SIZE;
        this.entireMessage = new byte[messageSize];
        linkData.slice().get(entireMessage);
        this.VNB = linkData.getInt();
        this.VNA = linkData.getInt();


        this.childDET = new byte[Constants.DET_SIZE];
        linkData.get(this.childDET);

        this.childHI = new byte[Constants.HI_SIZE];
        linkData.get(this.childHI);

        this.parentDET= new byte[Constants.DET_SIZE];
        linkData.get(this.parentDET);

        this.signature = new byte[Constants.SIGNATURE_SIZE];
        linkData.get(this.signature);
        this.isVerified = Boolean.FALSE;
    }

    public void verifySignature(byte[] parentHI){

        boolean signatureValid = Ed25519.verify(
                signature, 0, parentHI, 0, entireMessage, 0, messageSize);

        if(!signatureValid) {
            Log.e(TAG, "The signature verification was not successful");
            StringWriter sw = new StringWriter();
            for(byte b : entireMessage) {
                sw.append(String.format("%02X ", b));
            }
            Log.d(TAG, "Entire message: " + sw);
            sw=new StringWriter();

            for(byte b: signature){
                sw.append(String.format("%02X ", b));
            }
            Log.d(TAG,"Signature: "+sw);

            sw=new StringWriter();
            for(byte b: parentHI){
                sw.append(String.format("%02X ", b));
            }
            Log.d(TAG,"Public key: "+sw);
            return;
        }
        else {
            Log.d(TAG,"The signature is verified");
            this.isVerified=Boolean.TRUE;
            this.parentHI=parentHI;
        }
    }

    @Override
    public int hashCode() {
        return Objects.hash(VNB, VNA, Arrays.hashCode(childDET), Arrays.hashCode(parentDET), Arrays.hashCode(signature), Arrays.hashCode(childHI), Arrays.hashCode(parentHI), Arrays.hashCode(entireMessage), messageSize);
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if(this == obj) return true;
        if(obj == null || getClass() != obj.getClass()) return false;
        DRIPLinkObject msg= (DRIPLinkObject) obj;
        return Objects.equals(VNB,msg.VNB) &&
                Objects.equals(VNA,msg.VNA) &&
                Arrays.equals(childDET,msg.childDET) &&
                Arrays.equals(childHI,msg.childHI) &&
                Arrays.equals(parentDET,msg.parentDET) &&
                Arrays.equals(signature,msg.signature);
    }
}
